import os
import pkg_resources

import numpy as np
import pandas as pd

from hotspell.heatwaves import get_heatwaves
from hotspell.indices import index


def test_output_custom_index():
    filename = pkg_resources.resource_filename(
        "hotspell", os.path.join("datasets", "test_input.csv"),
    )

    hw_index = index(var="tmax", pct=90, min_duration=3, window_length=3,)

    heatwaves = get_heatwaves(
        filename=filename,
        hw_index=hw_index,
        ref_years=("1970-01-01", "1971-12-31"),
        export=False,
        metrics=False,
    )
    hw_events = heatwaves.events.iloc[:, 2:].astype(float).values

    input_file = pkg_resources.resource_filename(
        "hotspell", os.path.join("datasets", "target_output.csv"),
    )
    target_output = pd.read_csv(
        input_file, sep=",", skiprows=1, header=None, index_col=False
    )
    target_output = target_output.iloc[:, 2:].astype(float).values

    assert np.array_equal(hw_events, target_output) is True


def test_output_fixed_thres():
    filename = pkg_resources.resource_filename(
        "hotspell", os.path.join("datasets", "test_input.csv"),
    )

    hw_index = index(var="tmax", fixed_thres=38, min_duration=2)

    heatwaves = get_heatwaves(
        filename=filename,
        hw_index=hw_index,
        ref_years=("1970-01-01", "1971-12-31"),
        export=False,
        metrics=False,
    )
    hw_events = heatwaves.events.iloc[:, 2:].astype(float).values

    input_file = pkg_resources.resource_filename(
        "hotspell", os.path.join("datasets", "target_output_fixed_thres.csv"),
    )
    target_output = pd.read_csv(
        input_file, sep=",", skiprows=1, header=None, index_col=False
    )
    target_output = target_output.iloc[:, 2:].astype(float).values

    assert np.array_equal(hw_events, target_output) is True


def test_output_predefined_index():
    filename = pkg_resources.resource_filename(
        "hotspell", os.path.join("datasets", "test_input.csv"),
    )

    index_name = "test_index"
    hw_index = index(name=index_name)

    heatwaves = get_heatwaves(
        filename=filename,
        hw_index=hw_index,
        ref_years=("1970-01-01", "1971-12-31"),
        max_missing_days_pct=100,
        export=False,
    )
    hw_events = heatwaves.events.iloc[:, 2:].astype(float).values

    input_file = pkg_resources.resource_filename(
        "hotspell", os.path.join("datasets", "target_output.csv"),
    )
    target_output = pd.read_csv(
        input_file, sep=",", skiprows=1, header=None, index_col=False
    )
    target_output = target_output.iloc[:, 2:].astype(float).values

    assert np.array_equal(hw_events, target_output) is True


def test_output_southern_hem_predefined_index():
    filename = pkg_resources.resource_filename(
        "hotspell", os.path.join("datasets", "test_input_southern_hem.csv"),
    )

    index_name = "test_index"
    hw_index = index(name=index_name)

    heatwaves = get_heatwaves(
        filename=filename,
        hw_index=hw_index,
        ref_years=("1970-01-01", "1971-12-31"),
        summer_months=(12, 1, 2),
        export=False,
        metrics=False,
    )
    hw_events = heatwaves.events.iloc[:, 2:].astype(float).values

    input_file = pkg_resources.resource_filename(
        "hotspell", os.path.join("datasets", "target_output_southern_hem.csv"),
    )
    target_output = pd.read_csv(
        input_file, sep=",", skiprows=1, header=None, index_col=False
    )
    target_output = target_output.iloc[:, 2:].astype(float).values

    assert np.array_equal(hw_events, target_output) is True



class HeatWaveIndex:
    """
    A class used to represent a heat wave index.

    Attributes
    ----------
    name : str
        The name of the index. For predefined indices it follows the naming
        conventions of Perkins & Alexander (2013)
    var : str, one of "tmin" or "tmax"
        The meteorological variable.
    pct : int
        The percentile used as a threshold.
    fixed_thres : int or float
        The absolute threshold of the meteorological value. If both pct and
        fixed_thres are set, pct has precedence over fixed_thres.
    min_duration : int
        The minimum number of consecutive days should last so that a warm event
        is considered a heat wave.
    window_length : int
        The total number of days that a moving window has when computing the
        percentile value for each day.
    """

    def __init__(
        self, name, var, pct, fixed_thres, min_duration, window_length
    ):
        self.name = name
        self.var = var
        self.pct = pct
        self.fixed_thres = fixed_thres
        self.min_duration = min_duration
        self.window_length = window_length


def index(
    name=None,
    var=None,
    pct=None,
    fixed_thres=None,
    min_duration=None,
    window_length=None,
):
    """
    Create a predefined or custom HeatWaveIndex object.

    Parameters
    ----------
    name : str
        The name of the index. For predefined indices it follows the naming
        conventions of Perkins & Alexander (2013)
    var : str
        The meteorological variable.
    pct : int
        The percentile used as a threshold.
    fixed_thres : int or float
        The absolute threshold of the meteorological value. If both pct and
        fixed_thres are set, pct has precedence over fixed_thres.
    min_duration : int
        The minimum number of consecutive days should last so that a warm event
        is considered a heat wave.
    window_length : int
        The total number of days that a moving window has when computing the
        percentile value for each day.

    Returns
    -------
    HeatWaveIndex object
    """
    if name == "ctn90pct":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmin",
            pct=90,
            fixed_thres=None,
            min_duration=3,
            window_length=15,
        )
    elif name == "ctn95pct":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmin",
            pct=95,
            fixed_thres=None,
            min_duration=3,
            window_length=15,
        )
    elif name == "ctx90pct":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmax",
            pct=90,
            fixed_thres=None,
            min_duration=3,
            window_length=15,
        )
    elif name == "ctx95pct":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmax",
            pct=95,
            fixed_thres=None,
            min_duration=3,
            window_length=15,
        )
    elif name == "hot_days":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmax",
            pct=None,
            fixed_thres=35,
            min_duration=1,
            window_length=1,
        )
    elif name == "hot_events_daytime":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmax",
            pct=None,
            fixed_thres=35,
            min_duration=3,
            window_length=1,
        )
    elif name == "hot_events_nighttime":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmin",
            pct=None,
            fixed_thres=20,
            min_duration=3,
            window_length=1,
        )
    elif name == "summer_days":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmax",
            pct=None,
            fixed_thres=25,
            min_duration=1,
            window_length=1,
        )
    elif name == "tn90p":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmin",
            pct=90,
            fixed_thres=None,
            min_duration=1,
            window_length=5,
        )
    elif name == "tropical_nights":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmin",
            pct=None,
            fixed_thres=20,
            min_duration=1,
            window_length=1,
        )
    elif name == "tx90p":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmax",
            pct=90,
            fixed_thres=None,
            min_duration=1,
            window_length=5,
        )
    elif name == "wsdi":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmax",
            pct=90,
            fixed_thres=None,
            min_duration=6,
            window_length=5,
        )
    elif name == "test_index":
        hw_index = HeatWaveIndex(
            name=name,
            var="tmax",
            pct=90,
            fixed_thres=None,
            min_duration=3,
            window_length=3,
        )
    else:
        hw_index = HeatWaveIndex(
            name=name,
            var=var,
            pct=pct,
            fixed_thres=fixed_thres,
            min_duration=min_duration,
            window_length=window_length,
        )

    if hw_index.name is None:
        hw_index.name = "custom"

    if hw_index.window_length is None:
        hw_index.window_length = 1

    return hw_index



from pathlib import Path

from setuptools import setup

this_directory = Path(__file__).parent
long_description = (this_directory / "README.rst").read_text()

setup(
    name="hotspell",
    version="0.1.5.6",
    description="Detect heat waves from weather station data",
    author="Ilias Agathangelidis",
    packages=["hotspell"],
    package_data={"hotspell": ["datasets/*.pickle"]},
    install_requires=["numpy", "pandas"],
    long_description=long_description,
    long_description_content_type="text/x-rst",
    include_package_data=True,
    url="https://github.com/agathangelidis/hotspell",
)
